@@ -5,15 +5,16 @@ import os
 import tarfile
 import tempfile
 import time
-from typing import Dict, List, Union
+from typing import Dict, List, Optional, Union
 
 import requests
 import yaml
+from truss.truss_handle import TrussHandle
 
 from baseten.common import api
 from baseten.common.core import raises_api_error
 from baseten.common.lib_support import (build_h5_data_object,
-                                        coerce_input_to_json)
+                                        coerce_input_to_json, version_gte)
 
 logger = logging.getLogger(__name__)
 
@@ -58,21 +59,35 @@ def requirements_status(requirement_id: str):
 class BasetenDeployedModel:
     """A model backed by baseten serving. Provide either the model_id or the model_version_id."""
 
+    @raises_api_error
     def __init__(self, model_id: str = None,
                  model_version_id: str = None,
+                 external_model_version_id: str = None,
                  model_name: str = None,
-                 model_zoo_name: str = None):
-        if not model_id and not model_version_id:
-            raise ValueError('Either model_id or model_version_id must be provided.')
+                 model_zoo_name: str = None,
+                 truss_handle: TrussHandle = None,
+                 truss_spec_version: str = None):
+        if not model_id and not model_version_id and not external_model_version_id:
+            raise ValueError('Either model_id, model_version_id, or external_model_version_id must be provided.')
 
-        if model_id and model_version_id:
-            raise ValueError('Must provide either model_id or model_version_id; not both.')
+        if (
+            model_id
+            and (model_version_id or external_model_version_id)
+            or (external_model_version_id and model_version_id)
+        ):
+            raise ValueError(
+                'Must provide only one of model_id, model_version_id, or external_model_version_id; not more.')
 
         self._model_id = model_id
         self._model_version_id = model_version_id
+        self._external_model_version_id = external_model_version_id
+        if self._external_model_version_id:
+            self._model_version_id = api.model_version_external_id_get_version(self._external_model_version_id)
+
         self._model_name = model_name
-        self._model_config = None
+        self._truss_spec_version = truss_spec_version
         self.model_zoo_name = model_zoo_name
+        self.truss_handle = truss_handle
 
     @property
     def model_version_id(self):
@@ -99,6 +114,15 @@ class BasetenDeployedModel:
             TypeError: If the provided inputs is not of a supported type.
             ApiError: If there was an error communicating with the server.
         """
+        if self._truss_spec_version is None:
+            self._truss_spec_version = self.get_truss_spec_version()
+
+        if version_gte(self._truss_spec_version, "2.0"):
+            if self._model_version_id:
+                return api.predict_for_model_version(
+                    self._model_version_id, inputs, v2=True
+                )
+            return api.predict_for_model(self._model_id, inputs, v2=True)
 
         inputs_list, metadata = coerce_input_to_json(inputs, metadata)
 
@@ -129,6 +153,26 @@ class BasetenDeployedModel:
             raise ValueError('Only a BasetenDeployedModel backed by a model_version can be set as primary.')
         return api.set_primary(self._model_version_id)
 
+    @raises_api_error
+    def deactivate(self):
+        """Deactivate this version of the model.
+        Raises:
+            ApiError: If there was an error communicating with the server.
+        """
+        if not self._model_version_id:
+            raise ValueError('Only a BasetenDeployedModel backed by a model_version can be set as primary.')
+        return api.deactivate_model_version(self._model_version_id)
+
+    @raises_api_error
+    def activate(self):
+        """Activate this version of the model.
+        Raises:
+            ApiError: If there was an error communicating with the server.
+        """
+        if not self._model_version_id:
+            raise ValueError('Only a BasetenDeployedModel backed by a model_version can be set as primary.')
+        return api.activate_model_version(self._model_version_id)
+
     @raises_api_error
     def upload_sample_data(self,
                            feature_data,
@@ -232,6 +276,17 @@ class BasetenDeployedModel:
 
         raise ValueError('Please use on a BasetenDeployedModel instantiated with a model_version_id or model_id.')
 
+    @raises_api_error
+    def get_truss_spec_version(self) -> dict:
+        if self._model_id:
+            return api.model_truss_spec_version(self._model_id)
+        elif self._model_version_id:
+            return api.model_version_truss_spec_version(self._model_version_id)
+        elif self._external_model_version_id:
+            return api.external_version_truss_spec_version(self._external_model_version_id)
+
+        raise ValueError("Please use on a BasetenDeployedModel instantiated with a model_version_id or model_id.")
+
     @raises_api_error
     def add_artifact(
         self,
@@ -271,6 +326,10 @@ class BasetenDeployedModel:
             t.extractall(os.path.join(directory, self.model_version_id))
         return response
 
+    @property
+    def truss(self) -> Optional[TrussHandle]:
+        return self.truss_handle
+
 
 @raises_api_error
 def models_summary():
