@@ -1,4 +1,3 @@
-import configparser
 import functools
 import json
 import logging
@@ -19,21 +18,66 @@ from baseten.common.util import base64_encoded_json_str
 logger = logging.getLogger(__name__)
 
 
+class AuthToken:
+    value: str
+
+    def __init__(self, value: str):
+        self.value = value
+
+    def headers(self):
+        raise NotImplementedError
+
+
+class ApiKey(AuthToken):
+    def headers(self):
+        return {'Authorization': f'Api-Key {self.value}'}
+
+
+class JWT(AuthToken):
+    def headers(self):
+        return {"Authorization": self.value}
+
+
 def with_api_key(func):
     @functools.wraps(func)
     def wrapper(*args, **kwargs):
         config = settings.read_config()
-        try:
+
+        if config.has_option('api', 'api_key'):
             api_key = config.get('api', 'api_key')
-        except configparser.NoOptionError:
+        elif config.has_option('api', 'jwt_auth_header'):
+            raise AuthorizationError('This operation is not supported in the Web IDE')
+        else:
             raise AuthorizationError('You must first run the `baseten login` cli command.')
-        result = func(api_key, *args, **kwargs)
+
+        result = func(ApiKey(api_key), *args, **kwargs)
         return result
     return wrapper
 
 
-@with_api_key
-def models(api_key):
+def with_api_key_or_jwt(func):
+    """
+    Allows an API method to be called internally (with JWT) or externally
+    (with API Key).
+    """
+    @functools.wraps(func)
+    def wrapper(*args, **kwargs):
+        config = settings.read_config()
+
+        if config.has_option('api', 'api_key'):
+            result = func(ApiKey(config.get('api', 'api_key')), *args, **kwargs)
+            return result
+        elif config.has_option('api', 'jwt_auth_header'):
+            result = func(JWT(config.get('api', 'jwt_auth_header')), *args, **kwargs)
+            return result
+        else:
+            raise AuthorizationError('You must first run the `baseten login` cli command.')
+
+    return wrapper
+
+
+@with_api_key_or_jwt
+def models(auth_token: AuthToken):
     query_string = '''
     {
       models {
@@ -41,13 +85,15 @@ def models(api_key):
         name
         versions{
             id,
-            semver
+            semver,
+            current_deployment_status,
+            is_primary,
         }
       }
     }
     '''
 
-    resp = _post_graphql_query(api_key, query_string)
+    resp = _post_graphql_query(auth_token, query_string)
     return resp['data']
 
 
@@ -61,6 +107,9 @@ def get_model(api_key, model_name):
             versions{{
                 id
                 semver
+                truss_hash
+                truss_signature
+                is_draft
             }}
         }}
       }}
@@ -100,6 +149,21 @@ def pretrained_models(api_key):
     return resp['data']
 
 
+@with_api_key_or_jwt
+def pretrained_model_version(auth_token: AuthToken, pretrained_model_name: str):
+    query_string = f'''
+    {{
+      pretrained_model_version(pretrained_model_name: "{pretrained_model_name}"){{
+        model_id
+        current_deployment_status
+        truss_spec_version
+      }}
+    }}
+    '''
+    resp = _post_graphql_query(auth_token, query_string)
+    return resp['data']
+
+
 @with_api_key
 def models_summary(api_key):
     query_string = '''
@@ -121,6 +185,49 @@ def models_summary(api_key):
     return resp['data']['models']
 
 
+@with_api_key
+def model_version_external_id_get_version(api_key, external_model_version_id):
+    query_string = f"""
+    {{
+      model_version(external_model_version_id: "{external_model_version_id}") {{
+        id
+      }}
+    }}
+    """
+
+    resp = _post_graphql_query(api_key, query_string)
+    return resp["data"]["model_version"]["id"]
+
+
+@with_api_key_or_jwt
+def model_version_truss_spec_version(auth_token: AuthToken, model_version_id: str):
+    query_string = f"""
+    {{
+      model_version(id: "{model_version_id}") {{
+        truss_spec_version
+      }}
+    }}
+    """
+
+    resp = _post_graphql_query(auth_token, query_string)
+    return resp["data"]["model_version"]["truss_spec_version"]
+
+
+@with_api_key_or_jwt
+def model_truss_spec_version(auth_token: AuthToken, model_id: str):
+    query_string = f"""
+    {{
+      model(id: "{model_id}") {{
+        primary_version {{
+            truss_spec_version
+        }}
+      }}
+    }}
+    """
+    resp = _post_graphql_query(auth_token, query_string)
+    return resp["data"]["model"]["primary_version"]["truss_spec_version"]
+
+
 @with_api_key
 def create_model_from_truss(
     api_key,
@@ -130,6 +237,7 @@ def create_model_from_truss(
     semver_bump,
     client_version,
     is_trusted=False,
+    external_model_version_id=None,
 ):
     query_string = f'''
     mutation {{
@@ -137,8 +245,9 @@ def create_model_from_truss(
                    s3_key: "{s3_key}",
                    config: "{config}",
                    semver_bump: "{semver_bump}",
-                   client_version: "{client_version}"
+                   client_version: "{client_version}",
                    is_trusted: {'true' if is_trusted else 'false'}
+                   external_model_version_id: "{external_model_version_id if external_model_version_id else ''}"
 ) {{
         id,
         name,
@@ -158,6 +267,8 @@ def create_model_version_from_truss(
     config,
     semver_bump,
     client_version,
+    is_trusted=False,
+    external_model_version_id=None,
 ):
 
     query_string = f'''
@@ -168,6 +279,8 @@ def create_model_version_from_truss(
                    config: "{config}",
                    semver_bump: "{semver_bump}",
                    client_version: "{client_version}",
+                   is_trusted: {'true' if is_trusted else 'false'}
+                   external_model_version_id: "{external_model_version_id if external_model_version_id else ''}"
 
       ) {{
         id,
@@ -181,6 +294,58 @@ def create_model_version_from_truss(
     return resp['data']['create_model_version_from_truss']
 
 
+@with_api_key
+def deploy_draft_truss(
+    api_key,
+    model_name,
+    s3_key,
+    config,
+    client_version,
+    is_trusted=False,
+):
+    query_string = f'''
+    mutation {{
+      deploy_draft_truss(name: "{model_name}",
+                   s3_key: "{s3_key}",
+                   config: "{config}",
+                   client_version: "{client_version}",
+                   is_trusted: {'true' if is_trusted else 'false'},
+) {{
+        id,
+        name,
+        version_id
+      }}
+    }}
+    '''
+    resp = _post_graphql_query(api_key, query_string)
+    return resp['data']['deploy_draft_truss']
+
+
+@with_api_key
+def patch_draft_truss(
+    api_key,
+    model_name,
+    patch,
+    client_version,
+):
+    patch = base64_encoded_json_str(patch.to_dict())
+    query_string = f'''
+    mutation {{
+      patch_draft_truss(name: "{model_name}",
+                   client_version: "{client_version}",
+                   patch: "{patch}",
+) {{
+        id,
+        name,
+        version_id
+        succeeded
+      }}
+    }}
+    '''
+    resp = _post_graphql_query(api_key, query_string)
+    return resp['data']['patch_draft_truss']
+
+
 @with_api_key
 def create_model_configuration(
     api_key: str,
@@ -322,6 +487,26 @@ def signed_s3_upload_post(api_key, model_file_name):
     return resp['data']["signed_s3_upload_url"]
 
 
+@with_api_key
+def signed_s3_upload_and_download_post(api_key, file_name):
+    query_string = f'''
+    {{
+      signed_s3_upload_url(model_file_name: "{file_name}"){{
+        url,
+        get_url,
+        form_fields {{
+          key,
+          aws_access_key_id,
+          policy,
+          signature,
+        }}
+      }}
+    }}
+    '''
+    resp = _post_graphql_query(api_key, query_string)
+    return resp['data']["signed_s3_upload_url"]
+
+
 @with_api_key
 def get_model_s3_download_url(api_key, model_version_id):
     query_string = f'''
@@ -356,6 +541,27 @@ def artifact_signed_s3_upload_post(api_key, file_name, file_size):
     return resp['data']['artifact_signed_s3_upload_url']
 
 
+@with_api_key_or_jwt
+def user_files_signed_s3_upload_post(auth_token: AuthToken, file_name: str):
+    query_string = f'''
+    {{
+      user_files_signed_s3_upload_url(file_name: "{file_name}") {{
+        url,
+        get_url,
+        form_fields {{
+          key,
+          aws_access_key_id,
+          policy,
+          signature,
+        }}
+      }}
+    }}
+    '''
+
+    resp = _post_graphql_query(auth_token, query_string)
+    return resp['data']['user_files_signed_s3_upload_url']
+
+
 @with_api_key
 def register_data_for_model(api_key, s3_key, model_version_id, data_name):
     query_string = f'''
@@ -565,16 +771,17 @@ def model_version_linked_artifacts(
     return resp['data']['model_version']['artifacts']
 
 
-@with_api_key
-def predict_for_model(api_key,
+@with_api_key_or_jwt
+def predict_for_model(auth_token: AuthToken,
                       model_id: str,
-                      inputs: list,
+                      inputs: Union[List, Dict],
                       metadata: List[Dict] = None,
-                      prediction_only: bool = True) -> Union[List[List], Dict]:
+                      prediction_only: bool = True,
+                      v2: bool = False) -> Union[List[List], Dict]:
     """Call the model's predict given the input json.
 
     Args:
-        api_key (str)
+        auth_token (AuthToken)
         model_id (str)
         inputs (list)
         metadata (List[Dict]): Metadata key/value pairs (e.g. name, url), one for each input.
@@ -583,16 +790,24 @@ def predict_for_model(api_key,
     Raises:
         RequestException: If there was an error communicating with the server.
     """
-    predict_url = f'{settings.API_URL_BASE}/models/{model_id}/predict'
-    return _predict(api_key, predict_url, inputs, metadata, prediction_only)
+
+    # We have a special predict endpoint for internal use that we use
+    # if a JWT is present.
+    if isinstance(auth_token, ApiKey):
+        predict_url = f'{settings.get_server_url()}/models/{model_id}/predict'
+    elif isinstance(auth_token, JWT):
+        predict_url = f'{settings.get_server_url()}/models/{model_id}/predict_internal'
+
+    return _predict(auth_token, predict_url, inputs, metadata, prediction_only, v2=v2)
 
 
 @with_api_key
 def predict_for_model_version(api_key,
                               model_version_id: str,
-                              inputs: list,
+                              inputs: Union[List, Dict],
                               metadata: List[Dict] = None,
-                              prediction_only: bool = True) -> Union[List[List], Dict]:
+                              prediction_only: bool = True,
+                              v2: bool = False) -> Union[List[List], Dict]:
     """Call the model version's predict given the input json.
 
     Args:
@@ -604,8 +819,8 @@ def predict_for_model_version(api_key,
     Raises:
         RequestException: If there was an error communicating with the server.
     """
-    predict_url = f'{settings.API_URL_BASE}/model_versions/{model_version_id}/predict'
-    return _predict(api_key, predict_url, inputs, metadata, prediction_only)
+    predict_url = f'{settings.get_server_url()}/model_versions/{model_version_id}/predict'
+    return _predict(api_key, predict_url, inputs, metadata, prediction_only, v2=v2)
 
 
 @with_api_key
@@ -686,22 +901,45 @@ def requirement_status(api_key, requirement_id):
     return resp['data']['pynode_requirement']
 
 
-def _predict(api_key,
-             predict_url: str,
-             inputs: list,
-             metadata: List[Dict] = None,
-             prediction_only: bool = True) -> Union[List[List], Dict]:
-    resp = _post_rest_query(api_key, predict_url, {'inputs': inputs, 'metadata': metadata})
+def _format_inputs_for_v1(auth_token: AuthToken, inputs: Union[List, Dict], metadata: List[Dict]):
+    """
+    For internal (JWT) token use-cases, we do not need to package up the inputs.
+    For external cases, for the v1 model serving path, we expect inputs to be
+    provided in the format { inputs: ..., metadata: ... }.
+    """
+    if isinstance(auth_token, ApiKey):
+        return {'inputs': inputs, 'metadata': metadata}
+    else:
+        return inputs
+
+
+def _predict(
+    auth_token: AuthToken,
+    predict_url: str,
+    inputs: Union[List, Dict],
+    metadata: List[Dict] = None,
+    prediction_only: bool = True,
+    v2: bool = False
+):
+    if v2:
+        resp = _post_rest_query(auth_token, predict_url, inputs)
+        resp_json = json.loads(resp.content)
+        return resp_json["model_output"]
+
+    resp = _post_rest_query(
+        auth_token,
+        predict_url,
+        _format_inputs_for_v1(auth_token, inputs, metadata)
+    )
     resp_json = json.loads(resp.content)
     return resp_json['predictions'] if prediction_only else resp_json
 
 
-def _headers(api_key):
-    return {'Authorization': f'Api-Key {api_key}'}
+def _post_graphql_query(auth_token: AuthToken, query_string: str) -> dict:
+    headers = auth_token.headers()
 
-
-def _post_graphql_query(api_key, query_string) -> dict:
-    resp = requests.post(f'{settings.API_URL_BASE}/graphql/', data={'query': query_string}, headers=_headers(api_key))
+    resp = requests.post(f'{settings.get_server_url()}/graphql/',
+                         data={'query': query_string}, headers=headers)
     if not resp.ok:
         logger.error(f'GraphQL endpoint failed with error: {resp.content}')
         resp.raise_for_status()
@@ -712,8 +950,10 @@ def _post_graphql_query(api_key, query_string) -> dict:
     return resp_dict
 
 
-def _post_rest_query(api_key, url, post_body_dict):
-    resp = requests.post(url, json=post_body_dict, headers=_headers(api_key))
+def _post_rest_query(auth_token: AuthToken, url: str, post_body_dict: dict):
+    headers = auth_token.headers()
+
+    resp = requests.post(url, json=post_body_dict, headers=headers)
     resp.raise_for_status()
     return resp
 
@@ -769,6 +1009,19 @@ def upload_artifact(serialize_file: IO, file_ext: str, file_name: str, file_size
     return _upload_file(this_signed_s3_upload_post, form_fields)
 
 
+def upload_user_file(io_stream: IO, file_name: str) -> str:
+    signed_s3_upload_post = user_files_signed_s3_upload_post(file_name)
+    logger.debug(f'Signed s3 upload post:\n{json.dumps(signed_s3_upload_post, indent=4)}')
+
+    form_fields = signed_s3_upload_post['form_fields']
+    form_fields['AWSAccessKeyId'] = form_fields.pop('aws_access_key_id')  # S3 expects key name AWSAccessKeyId
+    form_fields['file'] = (file_name, io_stream)
+
+    _upload_file(signed_s3_upload_post, form_fields)
+
+    return signed_s3_upload_post["get_url"]
+
+
 def _upload_file(this_signed_s3_upload_post: dict, form_fields: dict) -> str:
     encoder = MultipartEncoder(fields=form_fields)
     encoder_len = encoder.len
@@ -808,3 +1061,61 @@ def serialize_artifact_to_s3(name, list_of_files):
     logger.info('Making contact with BaseTen 👋 👽')
     s3_key = upload_artifact(artifact_tgz, 'tgz', ARTIFACT_FILENAME, os.path.getsize(artifact_tgz.name))
     return s3_key
+
+
+@with_api_key
+def deactivate_model_version(api_key, model_version_id):
+    query_string = f'''
+    mutation {{
+        deactivate_model_version(model_version_id: "{model_version_id}") {{
+          ok
+        }}
+    }}
+    '''
+    resp = _post_graphql_query(api_key, query_string)
+    return resp['data']['deactivate_model_version']['ok']
+
+
+@with_api_key
+def activate_model_version(api_key, model_version_id):
+    query_string = f'''
+    mutation {{
+        activate_model_version(model_version_id: "{model_version_id}") {{
+          ok
+        }}
+    }}
+    '''
+    resp = _post_graphql_query(api_key, query_string)
+    return resp['data']['activate_model_version']['ok']
+
+
+@with_api_key_or_jwt
+def create_pretrained_model(auth_token: AuthToken, model_zoo_name: str, model_name: str):
+    query_string = f'''
+    mutation {{
+        create_pretrained_model(
+            model_zoo_name: "{model_zoo_name}",
+            name: "{model_name}",
+            description : "",
+            create_app: false
+        ) {{
+            model {{
+                created
+                id
+                name
+                platformType: model_platform_type
+                numberOfVersions: number_of_versions
+                description: description
+                primaryVersion: primary_version {{
+                    id
+                    model_id
+                    modelFramework: model_framework_display_name
+                    truss_spec_version
+                }}
+            }}
+        }}
+    }}
+    '''
+
+    resp = _post_graphql_query(auth_token, query_string)
+    return resp["data"]["create_pretrained_model"]["model"]
