@@ -1,4 +1,5 @@
 import functools
+import json
 from enum import Enum
 
 from requests import RequestException
@@ -61,7 +62,13 @@ class ApiError(Error):
     def __str__(self):
         error_str = self.message
         if self.response is not None:  # non-200 Response objects are falsy, hence the not None.
-            error_str = f'{error_str}\n<Server response: {self.response.content}>'
+            error_message = json.loads(self.response.content)
+            error_message = (
+                error_message["error"]
+                if "error" in error_message
+                else error_message
+            )
+            error_str = f"{error_str}\n<Server response: {error_message}>"
         return error_str
 
 
